@@ -23,6 +23,10 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#if defined(GDK_PIXBUF_DISABLE_SINGLE_INCLUDES) && !defined (GDK_PIXBUF_H_INSIDE) && !defined (GDK_PIXBUF_COMPILATION)
+#error "Only <gdk-pixbuf/gdk-pixbuf.h> can be included directly."
+#endif
+
 #ifndef GDK_PIXBUF_CORE_H
 #define GDK_PIXBUF_CORE_H
 
