@@ -142,56 +142,56 @@ gtk_list_item_class_init (GtkListItemClass *class)
   class->toggle_add_mode = NULL;
 
   list_item_signals[TOGGLE_FOCUS_ROW] =
-    gtk_signal_new (I_("toggle_focus_row"),
+    gtk_signal_new (I_("toggle-focus-row"),
                     GTK_RUN_LAST | GTK_RUN_ACTION,
                     GTK_CLASS_TYPE (object_class),
                     GTK_SIGNAL_OFFSET (GtkListItemClass, toggle_focus_row),
                     _gtk_marshal_VOID__VOID,
                     GTK_TYPE_NONE, 0);
   list_item_signals[SELECT_ALL] =
-    gtk_signal_new (I_("select_all"),
+    gtk_signal_new (I_("select-all"),
                     GTK_RUN_LAST | GTK_RUN_ACTION,
                     GTK_CLASS_TYPE (object_class),
                     GTK_SIGNAL_OFFSET (GtkListItemClass, select_all),
                     _gtk_marshal_VOID__VOID,
                     GTK_TYPE_NONE, 0);
   list_item_signals[UNSELECT_ALL] =
-    gtk_signal_new (I_("unselect_all"),
+    gtk_signal_new (I_("unselect-all"),
                     GTK_RUN_LAST | GTK_RUN_ACTION,
                     GTK_CLASS_TYPE (object_class),
                     GTK_SIGNAL_OFFSET (GtkListItemClass, unselect_all),
                     _gtk_marshal_VOID__VOID,
                     GTK_TYPE_NONE, 0);
   list_item_signals[UNDO_SELECTION] =
-    gtk_signal_new (I_("undo_selection"),
+    gtk_signal_new (I_("undo-selection"),
 		    GTK_RUN_LAST | GTK_RUN_ACTION,
 		    GTK_CLASS_TYPE (object_class),
 		    GTK_SIGNAL_OFFSET (GtkListItemClass, undo_selection),
 		    _gtk_marshal_VOID__VOID,
 		    GTK_TYPE_NONE, 0);
   list_item_signals[START_SELECTION] =
-    gtk_signal_new (I_("start_selection"),
+    gtk_signal_new (I_("start-selection"),
 		    GTK_RUN_LAST | GTK_RUN_ACTION,
 		    GTK_CLASS_TYPE (object_class),
 		    GTK_SIGNAL_OFFSET (GtkListItemClass, start_selection),
 		    _gtk_marshal_VOID__VOID,
 		    GTK_TYPE_NONE, 0);
   list_item_signals[END_SELECTION] =
-    gtk_signal_new (I_("end_selection"),
+    gtk_signal_new (I_("end-selection"),
 		    GTK_RUN_LAST | GTK_RUN_ACTION,
 		    GTK_CLASS_TYPE (object_class),
 		    GTK_SIGNAL_OFFSET (GtkListItemClass, end_selection),
 		    _gtk_marshal_VOID__VOID,
 		    GTK_TYPE_NONE, 0);
   list_item_signals[TOGGLE_ADD_MODE] =
-    gtk_signal_new (I_("toggle_add_mode"),
+    gtk_signal_new (I_("toggle-add-mode"),
 		    GTK_RUN_LAST | GTK_RUN_ACTION,
 		    GTK_CLASS_TYPE (object_class),
 		    GTK_SIGNAL_OFFSET (GtkListItemClass, toggle_add_mode),
 		    _gtk_marshal_VOID__VOID,
 		    GTK_TYPE_NONE, 0);
   list_item_signals[EXTEND_SELECTION] =
-    gtk_signal_new (I_("extend_selection"),
+    gtk_signal_new (I_("extend-selection"),
                     GTK_RUN_LAST | GTK_RUN_ACTION,
                     GTK_CLASS_TYPE (object_class),
                     GTK_SIGNAL_OFFSET (GtkListItemClass, extend_selection),
@@ -199,14 +199,14 @@ gtk_list_item_class_init (GtkListItemClass *class)
                     GTK_TYPE_NONE, 3,
 		    GTK_TYPE_SCROLL_TYPE, GTK_TYPE_FLOAT, GTK_TYPE_BOOL);
   list_item_signals[SCROLL_VERTICAL] =
-    gtk_signal_new (I_("scroll_vertical"),
+    gtk_signal_new (I_("scroll-vertical"),
                     GTK_RUN_LAST | GTK_RUN_ACTION,
                     GTK_CLASS_TYPE (object_class),
                     GTK_SIGNAL_OFFSET (GtkListItemClass, scroll_vertical),
                     _gtk_marshal_VOID__ENUM_FLOAT,
                     GTK_TYPE_NONE, 2, GTK_TYPE_SCROLL_TYPE, GTK_TYPE_FLOAT);
   list_item_signals[SCROLL_HORIZONTAL] =
-    gtk_signal_new (I_("scroll_horizontal"),
+    gtk_signal_new (I_("scroll-horizontal"),
                     GTK_RUN_LAST | GTK_RUN_ACTION,
                     GTK_CLASS_TYPE (object_class),
                     GTK_SIGNAL_OFFSET (GtkListItemClass, scroll_horizontal),
@@ -215,168 +215,168 @@ gtk_list_item_class_init (GtkListItemClass *class)
 
   binding_set = gtk_binding_set_by_class (class);
   gtk_binding_entry_add_signal (binding_set, GDK_Up, 0,
-				"scroll_vertical", 2,
+				"scroll-vertical", 2,
 				GTK_TYPE_ENUM, GTK_SCROLL_STEP_BACKWARD,
 				GTK_TYPE_FLOAT, 0.0);
   gtk_binding_entry_add_signal (binding_set, GDK_KP_Up, 0,
-				"scroll_vertical", 2,
+				"scroll-vertical", 2,
 				GTK_TYPE_ENUM, GTK_SCROLL_STEP_BACKWARD,
 				GTK_TYPE_FLOAT, 0.0);
   gtk_binding_entry_add_signal (binding_set, GDK_Down, 0,
-				"scroll_vertical", 2,
+				"scroll-vertical", 2,
 				GTK_TYPE_ENUM, GTK_SCROLL_STEP_FORWARD,
 				GTK_TYPE_FLOAT, 0.0);
   gtk_binding_entry_add_signal (binding_set, GDK_KP_Down, 0,
-				"scroll_vertical", 2,
+				"scroll-vertical", 2,
 				GTK_TYPE_ENUM, GTK_SCROLL_STEP_FORWARD,
 				GTK_TYPE_FLOAT, 0.0);
   gtk_binding_entry_add_signal (binding_set, GDK_Page_Up, 0,
-				"scroll_vertical", 2,
+				"scroll-vertical", 2,
 				GTK_TYPE_ENUM, GTK_SCROLL_PAGE_BACKWARD,
 				GTK_TYPE_FLOAT, 0.0);
   gtk_binding_entry_add_signal (binding_set, GDK_KP_Page_Up, 0,
-				"scroll_vertical", 2,
+				"scroll-vertical", 2,
 				GTK_TYPE_ENUM, GTK_SCROLL_PAGE_BACKWARD,
 				GTK_TYPE_FLOAT, 0.0);
   gtk_binding_entry_add_signal (binding_set, GDK_Page_Down, 0,
-				"scroll_vertical", 2,
+				"scroll-vertical", 2,
 				GTK_TYPE_ENUM, GTK_SCROLL_PAGE_FORWARD,
 				GTK_TYPE_FLOAT, 0.0);
   gtk_binding_entry_add_signal (binding_set, GDK_KP_Page_Down, 0,
-				"scroll_vertical", 2,
+				"scroll-vertical", 2,
 				GTK_TYPE_ENUM, GTK_SCROLL_PAGE_FORWARD,
 				GTK_TYPE_FLOAT, 0.0);
   gtk_binding_entry_add_signal (binding_set, GDK_Home, GDK_CONTROL_MASK,
-				"scroll_vertical", 2,
+				"scroll-vertical", 2,
 				GTK_TYPE_ENUM, GTK_SCROLL_JUMP,
 				GTK_TYPE_FLOAT, 0.0);
   gtk_binding_entry_add_signal (binding_set, GDK_KP_Home, GDK_CONTROL_MASK,
-				"scroll_vertical", 2,
+				"scroll-vertical", 2,
 				GTK_TYPE_ENUM, GTK_SCROLL_JUMP,
 				GTK_TYPE_FLOAT, 0.0);
   gtk_binding_entry_add_signal (binding_set, GDK_End, GDK_CONTROL_MASK,
-				"scroll_vertical", 2,
+				"scroll-vertical", 2,
 				GTK_TYPE_ENUM, GTK_SCROLL_JUMP,
 				GTK_TYPE_FLOAT, 1.0);
   gtk_binding_entry_add_signal (binding_set, GDK_KP_End, GDK_CONTROL_MASK,
-				"scroll_vertical", 2,
+				"scroll-vertical", 2,
 				GTK_TYPE_ENUM, GTK_SCROLL_JUMP,
 				GTK_TYPE_FLOAT, 1.0);
   
   gtk_binding_entry_add_signal (binding_set, GDK_Up, GDK_SHIFT_MASK,
-				"extend_selection", 3,
+				"extend-selection", 3,
 				GTK_TYPE_ENUM, GTK_SCROLL_STEP_BACKWARD,
 				GTK_TYPE_FLOAT, 0.0, GTK_TYPE_BOOL, TRUE);
   gtk_binding_entry_add_signal (binding_set, GDK_KP_Up, GDK_SHIFT_MASK,
-				"extend_selection", 3,
+				"extend-selection", 3,
 				GTK_TYPE_ENUM, GTK_SCROLL_STEP_BACKWARD,
 				GTK_TYPE_FLOAT, 0.0, GTK_TYPE_BOOL, TRUE);
   
   gtk_binding_entry_add_signal (binding_set, GDK_Down, GDK_SHIFT_MASK,
-				"extend_selection", 3,
+				"extend-selection", 3,
 				GTK_TYPE_ENUM, GTK_SCROLL_STEP_FORWARD,
 				GTK_TYPE_FLOAT, 0.0, GTK_TYPE_BOOL, TRUE);
   gtk_binding_entry_add_signal (binding_set, GDK_KP_Down, GDK_SHIFT_MASK,
-				"extend_selection", 3,
+				"extend-selection", 3,
 				GTK_TYPE_ENUM, GTK_SCROLL_STEP_FORWARD,
 				GTK_TYPE_FLOAT, 0.0, GTK_TYPE_BOOL, TRUE);
   gtk_binding_entry_add_signal (binding_set, GDK_Page_Up, GDK_SHIFT_MASK,
-				"extend_selection", 3,
+				"extend-selection", 3,
 				GTK_TYPE_ENUM, GTK_SCROLL_PAGE_BACKWARD,
 				GTK_TYPE_FLOAT, 0.0, GTK_TYPE_BOOL, TRUE);
   gtk_binding_entry_add_signal (binding_set, GDK_KP_Page_Up, GDK_SHIFT_MASK,
-				"extend_selection", 3,
+				"extend-selection", 3,
 				GTK_TYPE_ENUM, GTK_SCROLL_PAGE_BACKWARD,
 				GTK_TYPE_FLOAT, 0.0, GTK_TYPE_BOOL, TRUE);
   gtk_binding_entry_add_signal (binding_set, GDK_Page_Down, GDK_SHIFT_MASK,
-				"extend_selection", 3,
+				"extend-selection", 3,
 				GTK_TYPE_ENUM, GTK_SCROLL_PAGE_FORWARD,
 				GTK_TYPE_FLOAT, 0.0, GTK_TYPE_BOOL, TRUE);
   gtk_binding_entry_add_signal (binding_set, GDK_KP_Page_Down, GDK_SHIFT_MASK,
-				"extend_selection", 3,
+				"extend-selection", 3,
 				GTK_TYPE_ENUM, GTK_SCROLL_PAGE_FORWARD,
 				GTK_TYPE_FLOAT, 0.0, GTK_TYPE_BOOL, TRUE);
   gtk_binding_entry_add_signal (binding_set, GDK_Home,
 				GDK_SHIFT_MASK | GDK_CONTROL_MASK,
-				"extend_selection", 3,
+				"extend-selection", 3,
 				GTK_TYPE_ENUM, GTK_SCROLL_JUMP,
 				GTK_TYPE_FLOAT, 0.0, GTK_TYPE_BOOL, TRUE);
   gtk_binding_entry_add_signal (binding_set, GDK_KP_Home,
 				GDK_SHIFT_MASK | GDK_CONTROL_MASK,
-				"extend_selection", 3,
+				"extend-selection", 3,
 				GTK_TYPE_ENUM, GTK_SCROLL_JUMP,
 				GTK_TYPE_FLOAT, 0.0, GTK_TYPE_BOOL, TRUE);
   gtk_binding_entry_add_signal (binding_set, GDK_End,
 				GDK_SHIFT_MASK | GDK_CONTROL_MASK,
-				"extend_selection", 3,
+				"extend-selection", 3,
 				GTK_TYPE_ENUM, GTK_SCROLL_JUMP,
 				GTK_TYPE_FLOAT, 1.0, GTK_TYPE_BOOL, TRUE);
   gtk_binding_entry_add_signal (binding_set, GDK_KP_End,
 				GDK_SHIFT_MASK | GDK_CONTROL_MASK,
-				"extend_selection", 3,
+				"extend-selection", 3,
 				GTK_TYPE_ENUM, GTK_SCROLL_JUMP,
 				GTK_TYPE_FLOAT, 1.0, GTK_TYPE_BOOL, TRUE);
 
   
   gtk_binding_entry_add_signal (binding_set, GDK_Left, 0,
-				"scroll_horizontal", 2,
+				"scroll-horizontal", 2,
 				GTK_TYPE_ENUM, GTK_SCROLL_STEP_BACKWARD,
 				GTK_TYPE_FLOAT, 0.0);
   gtk_binding_entry_add_signal (binding_set, GDK_KP_Left, 0,
-				"scroll_horizontal", 2,
+				"scroll-horizontal", 2,
 				GTK_TYPE_ENUM, GTK_SCROLL_STEP_BACKWARD,
 				GTK_TYPE_FLOAT, 0.0);
   gtk_binding_entry_add_signal (binding_set, GDK_Right, 0,
-				"scroll_horizontal", 2,
+				"scroll-horizontal", 2,
 				GTK_TYPE_ENUM, GTK_SCROLL_STEP_FORWARD,
 				GTK_TYPE_FLOAT, 0.0);
   gtk_binding_entry_add_signal (binding_set, GDK_KP_Right, 0,
-				"scroll_horizontal", 2,
+				"scroll-horizontal", 2,
 				GTK_TYPE_ENUM, GTK_SCROLL_STEP_FORWARD,
 				GTK_TYPE_FLOAT, 0.0);
   gtk_binding_entry_add_signal (binding_set, GDK_Home, 0,
-				"scroll_horizontal", 2,
+				"scroll-horizontal", 2,
 				GTK_TYPE_ENUM, GTK_SCROLL_JUMP,
 				GTK_TYPE_FLOAT, 0.0);
   gtk_binding_entry_add_signal (binding_set, GDK_KP_Home, 0,
-				"scroll_horizontal", 2,
+				"scroll-horizontal", 2,
 				GTK_TYPE_ENUM, GTK_SCROLL_JUMP,
 				GTK_TYPE_FLOAT, 0.0);
   gtk_binding_entry_add_signal (binding_set, GDK_End, 0,
-				"scroll_horizontal", 2,
+				"scroll-horizontal", 2,
 				GTK_TYPE_ENUM, GTK_SCROLL_JUMP,
 				GTK_TYPE_FLOAT, 1.0);
   gtk_binding_entry_add_signal (binding_set, GDK_KP_End, 0,
-				"scroll_horizontal", 2,
+				"scroll-horizontal", 2,
 				GTK_TYPE_ENUM, GTK_SCROLL_JUMP,
 				GTK_TYPE_FLOAT, 1.0);
   
   gtk_binding_entry_add_signal (binding_set, GDK_Escape, 0,
-				"undo_selection", 0);
+				"undo-selection", 0);
   gtk_binding_entry_add_signal (binding_set, GDK_space, 0,
-				"toggle_focus_row", 0);
+				"toggle-focus-row", 0);
   gtk_binding_entry_add_signal (binding_set, GDK_KP_Space, 0,
-				"toggle_focus_row", 0);
+				"toggle-focus-row", 0);
   gtk_binding_entry_add_signal (binding_set, GDK_space, GDK_CONTROL_MASK,
-				"toggle_add_mode", 0);
+				"toggle-add-mode", 0);
   gtk_binding_entry_add_signal (binding_set, GDK_KP_Space, GDK_CONTROL_MASK,
-				"toggle_add_mode", 0);
+				"toggle-add-mode", 0);
   gtk_binding_entry_add_signal (binding_set, GDK_slash, GDK_CONTROL_MASK,
-				"select_all", 0);
+				"select-all", 0);
   gtk_binding_entry_add_signal (binding_set, GDK_KP_Divide, GDK_CONTROL_MASK,
-				"select_all", 0);
+				"select-all", 0);
   gtk_binding_entry_add_signal (binding_set, '\\', GDK_CONTROL_MASK,
-				"unselect_all", 0);
+				"unselect-all", 0);
   gtk_binding_entry_add_signal (binding_set, GDK_Shift_L,
 				GDK_RELEASE_MASK | GDK_SHIFT_MASK,
-				"end_selection", 0);
+				"end-selection", 0);
   gtk_binding_entry_add_signal (binding_set, GDK_Shift_R,
 				GDK_RELEASE_MASK | GDK_SHIFT_MASK,
-				"end_selection", 0);
+				"end-selection", 0);
   gtk_binding_entry_add_signal (binding_set, GDK_Shift_R,
 				GDK_RELEASE_MASK | GDK_SHIFT_MASK |
 				GDK_CONTROL_MASK,
-				"end_selection", 0);
+				"end-selection", 0);
 }
 
 static void
@@ -427,8 +427,7 @@ gtk_list_item_realize (GtkWidget *widget)
   GdkWindowAttr attributes;
   gint attributes_mask;
 
-  /*if (GTK_WIDGET_CLASS (parent_class)->realize)
-    (* GTK_WIDGET_CLASS (parent_class)->realize) (widget);*/
+  /*GTK_WIDGET_CLASS (parent_class)->realize (widget);*/
 
   g_return_if_fail (GTK_IS_LIST_ITEM (widget));
 
@@ -566,8 +565,8 @@ gtk_list_item_expose (GtkWidget      *widget,
                               0, 0, -1, -1);           
         }
 
-      (* GTK_WIDGET_CLASS (parent_class)->expose_event) (widget, event);
-      
+      GTK_WIDGET_CLASS (parent_class)->expose_event (widget, event);
+
       if (GTK_WIDGET_HAS_FOCUS (widget))
         {
           if (GTK_IS_LIST (widget->parent) && GTK_LIST (widget->parent)->add_mode)
