@@ -82,7 +82,8 @@ enum
   PROP_BLINKING,
   PROP_HAS_TOOLTIP,
   PROP_TOOLTIP_TEXT,
-  PROP_TOOLTIP_MARKUP
+  PROP_TOOLTIP_MARKUP,
+  PROP_TITLE
 };
 
 enum 
@@ -116,12 +117,14 @@ struct _GtkStatusIconPrivate
   gint		last_click_x, last_click_y;
   GtkOrientation orientation;
   gchar         *tooltip_text;
+  gchar         *title;
 #endif
 	
 #ifdef GDK_WINDOWING_QUARTZ
   GtkWidget     *dummy_widget;
   GtkQuartzStatusIcon *status_item;
   gchar         *tooltip_text;
+  gchar         *title;
 #endif
 
   gint          size;
@@ -367,6 +370,10 @@ gtk_status_icon_class_init (GtkStatusIconClass *class)
    * the default handler for the #GtkStatusIcon::query-tooltip signal
    * will take care of displaying the tooltip.
    *
+   * Note that some platforms have limitations on the length of tooltips
+   * that they allow on status icons, e.g. Windows only shows the first
+   * 64 characters.
+   *
    * Since: 2.16
    */
   g_object_class_install_property (gobject_class,
@@ -402,6 +409,23 @@ gtk_status_icon_class_init (GtkStatusIconClass *class)
 							GTK_PARAM_READWRITE));
 
 
+  /**
+   * GtkStatusIcon:title:
+   *
+   * The title of this tray icon. This should be a short, human-readable,
+   * localized string describing the tray icon. It may be used by tools
+   * like screen readers to render the tray icon.
+   *
+   * Since: 2.18
+   */
+  g_object_class_install_property (gobject_class,
+                                   PROP_TITLE,
+                                   g_param_spec_string ("title",
+                                                        P_("Title"),
+                                                        P_("The title of this tray icon"),
+                                                        NULL,
+                                                        GTK_PARAM_READWRITE));
+
   /**
    * GtkStatusIcon::activate:
    * @status_icon: the object which received the signal
@@ -942,6 +966,24 @@ gtk_status_icon_finalize (GObject *object)
   priv->blank_icon = NULL;
 
 #ifdef GDK_WINDOWING_X11
+  g_signal_handlers_disconnect_by_func (priv->tray_icon,
+			                gtk_status_icon_key_press, status_icon);
+  g_signal_handlers_disconnect_by_func (priv->tray_icon,
+			                gtk_status_icon_popup_menu, status_icon);
+  g_signal_handlers_disconnect_by_func (priv->tray_icon,
+			                gtk_status_icon_embedded_changed, status_icon);
+  g_signal_handlers_disconnect_by_func (priv->tray_icon,
+			                gtk_status_icon_orientation_changed, status_icon);
+  g_signal_handlers_disconnect_by_func (priv->tray_icon,
+			                gtk_status_icon_button_press, status_icon);
+  g_signal_handlers_disconnect_by_func (priv->tray_icon,
+			                gtk_status_icon_button_release, status_icon);
+  g_signal_handlers_disconnect_by_func (priv->tray_icon,
+			                gtk_status_icon_scroll, status_icon);
+  g_signal_handlers_disconnect_by_func (priv->tray_icon,
+			                gtk_status_icon_query_tooltip, status_icon);
+  g_signal_handlers_disconnect_by_func (priv->tray_icon,
+		    	                gtk_status_icon_screen_changed, status_icon);
   gtk_widget_destroy (priv->image);
   gtk_widget_destroy (priv->tray_icon);
 #endif
@@ -1004,10 +1046,16 @@ gtk_status_icon_set_property (GObject      *object,
       break;
     case PROP_HAS_TOOLTIP:
       gtk_status_icon_set_has_tooltip (status_icon, g_value_get_boolean (value));
+      break;
     case PROP_TOOLTIP_TEXT:
       gtk_status_icon_set_tooltip_text (status_icon, g_value_get_string (value));
+      break;
     case PROP_TOOLTIP_MARKUP:
       gtk_status_icon_set_tooltip_markup (status_icon, g_value_get_string (value));
+      break;
+    case PROP_TITLE:
+      gtk_status_icon_set_title (status_icon, g_value_get_string (value));
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -1090,6 +1138,9 @@ gtk_status_icon_get_property (GObject    *object,
     case PROP_TOOLTIP_MARKUP:
       g_value_set_string (value, gtk_status_icon_get_tooltip_markup (status_icon));
       break;
+    case PROP_TITLE:
+      g_value_set_string (value, gtk_status_icon_get_title (status_icon));
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -1762,7 +1813,8 @@ gtk_status_icon_reset_image_data (GtkStatusIcon *status_icon)
       break;
 
     case GTK_IMAGE_GICON:
-      g_free (priv->image_data.gicon);
+      if (priv->image_data.gicon)
+        g_object_unref (priv->image_data.gicon);
       priv->image_data.gicon = NULL;
 
       g_object_notify (G_OBJECT (status_icon), "gicon");
@@ -1931,6 +1983,9 @@ gtk_status_icon_set_from_gicon (GtkStatusIcon *status_icon,
   g_return_if_fail (GTK_IS_STATUS_ICON (status_icon));
   g_return_if_fail (icon != NULL);
 
+  if (icon)
+    g_object_ref (icon);
+
   gtk_status_icon_set_image (status_icon, GTK_IMAGE_GICON,
                              (gpointer) icon);
 }
@@ -2689,7 +2744,7 @@ gtk_status_icon_set_tooltip_text (GtkStatusIcon *status_icon,
 #endif
 #ifdef GDK_WINDOWING_QUARTZ
   QUARTZ_POOL_ALLOC;
-  [priv->status_item setToolTip:tooltip_text];
+  [priv->status_item setToolTip:text];
   QUARTZ_POOL_RELEASE;
 
   g_free (priv->tooltip_text);
@@ -2846,5 +2901,73 @@ gtk_status_icon_get_x11_window_id (GtkStatusIcon *status_icon)
 #endif
 }
 
+/**
+ * gtk_status_icon_set_title:
+ * @status_icon: a #GtkStatusIcon
+ * @title: the title 
+ *
+ * Sets the title of this tray icon.
+ * This should be a short, human-readable, localized string 
+ * describing the tray icon. It may be used by tools like screen
+ * readers to render the tray icon.
+ *
+ * Since: 2.18
+ */
+void
+gtk_status_icon_set_title (GtkStatusIcon *status_icon,
+                           const gchar   *title)
+{
+  GtkStatusIconPrivate *priv;
+
+  g_return_if_fail (GTK_IS_STATUS_ICON (status_icon));
+
+  priv = status_icon->priv;
+
+#ifdef GDK_WINDOWING_X11
+  gtk_window_set_title (GTK_WINDOW (priv->tray_icon), title);
+#endif
+#ifdef GDK_WINDOWING_QUARTZ
+  g_free (priv->title);
+  priv->title = g_strdup (title);
+#endif
+#ifdef GDK_WINDOWING_WIN32
+  g_free (priv->title);
+  priv->title = g_strdup (title);
+#endif
+
+  g_object_notify (G_OBJECT (status_icon), "title");
+}
+
+/**
+ * gtk_status_icon_get_title:
+ * @status_icon: a #GtkStatusIcon
+ *
+ * Gets the title of this tray icon. See gtk_status_icon_set_title().
+ *
+ * Returns: the title of the status icon
+ *
+ * Since: 2.18
+ */
+G_CONST_RETURN gchar *
+gtk_status_icon_get_title (GtkStatusIcon *status_icon)
+{
+  GtkStatusIconPrivate *priv;
+
+  g_return_val_if_fail (GTK_IS_STATUS_ICON (status_icon), NULL);
+
+  priv = status_icon->priv;
+
+#ifdef GDK_WINDOWING_X11
+  return gtk_window_get_title (GTK_WINDOW (priv->tray_icon));
+#endif
+#ifdef GDK_WINDOWING_QUARTZ
+  return priv->title;
+#endif
+#ifdef GDK_WINDOWING_WIN32
+  return priv->title;
+#endif
+}
+
+
 #define __GTK_STATUS_ICON_C__
 #include "gtkaliasdef.c"
